// This files groups functions related to external program handling
//
// class ExternalProgram
//	interruptible:bool	- do we have control over program execution ? (can we call interrupt)
//	takeAudio:bool		- does it needs control of audio device (release xine ?)
//	fullscreen:bool		- does it hides ost-box ?
//	controlLevel:int	- does ostbox still receive input while running ?
//				  	0 => normal (no grab, don't disable input)
//					1 => all usefull keys
//					5 => all excepted channel keys (back/power/volume)
//					10 => back key and power key only
//					255 => no key at all (default)
//	interrupting:bool	- INTERNAL : set to true when terminating
//
//	producer		- The corresponding producer (during execution). 
//				  Automatically deleted after use
//
//	interrupt: function	- return true if program is stopped, false if failed. No handler is called
//
//	interrupted: function	- called when the program got interrupted (after a successful call to interrupt)
//				  must not start another externalProgram
//
//	terminated: function	- called when the program terminated normally


// Consutrctor of the ExternalProgram class
function ExternalProgram(cmd)
{
    this.interruptible = false;
    this.takeAudio = true;
    this.fullscreen = true;
    this.controlLevel = 255;
    this.interrupting = false;
    this.terminatedFlag = false;
    if (cmd != undefined) {
	this.producer = newProducer(cmd);
    } else {
	this.producer = undefined;
    }
    this.interrupt = function() { return false; }
    this.interrupted = function() {}
    this.terminated = function() {}
}

var currentExternalProgram = undefined;


// Run a program in fullscreen with all key shortcut disabled
// Return false if this is not possible at the moment
function runFullscreenProgram(cmd)
{
    return runProgram(new ExternalProgram(cmd));
}

// terminate a current program, but don't leave its state
function terminateCurrentProgram(funcKill)
{
    if (currentExternalProgram == undefined) return true;

    currentExternalProgram.interrupting = true;
    
    if (funcKill()) {
	print("killCurrentProgram : funcKill succeded\n");
	currentExternalProgram.terminatedFlag = true;
	if (currentExternalProgram.producer != undefined) {
	    currentExternalProgram.producer.cancel();
	    currentExternalProgram.producer = undefined;
	}
	return true;
    } else {
	print("killCurrentProgram : funcKill failed\n");
	currentExternalProgram.interrupting = false;
	return false;
    }
}

// Run a program described by an object
// progObject may be undefined. That stops running program
function runProgram(progObject)
{
    // First, try to terminate current program
    if (currentExternalProgram != undefined) {
	var oldExternalProgram = currentExternalProgram;
	
	if (!currentExternalProgram.terminatedFlag) {
	    currentExternalProgram.interrupting = true;
	    if (!currentExternalProgram.interrupt()) {
		currentExternalProgram.interrupting = false;
		return false;
	    }
	    
	    oldExternalProgram.interrupted();
	}
	
	if (oldExternalProgram.producer != undefined) {
	    oldExternalProgram.producer.cancel();
	}
    }
    
    switchStateFor(progObject);
    
    currentExternalProgram = progObject;
    
    if ((currentExternalProgram != undefined) && (currentExternalProgram.producer != undefined)) {
	currentExternalProgram.producer.externalProgram = currentExternalProgram;
	currentExternalProgram.producer.childExited = function() {
	    debug("currentExternalProgram.producer.childExited\n");
	    if (!this.externalProgram.interrupting) {
		externalProgramTerminated(this.externalProgram);
	    }
	}
	
	currentExternalProgram.producer.start();
    }
    return true;
}

function externalProgramTerminated(progObject)
{
    debug("externalProgramTerminated !\n");
    if (progObject.producer != undefined) {
	progObject.producer.cancel();
    }
    
    if (progObject != currentExternalProgram) {
	debug("not the right one...\n");
	return;
    }
    switchStateFor(undefined);
    currentExternalProgram = undefined;
    progObject.terminated();
}

function switchStateFor(progObject)
{
    // State of current running program
    var currentAudioState = false;
    // State of current running progra
    var currentFullscreenState = false;
    // State of current running program
    var currentControlLevel = 0;
    
    var newAudioState = false;
    var newFullscreenState = false;
    var newControlLevel = 0;
    
    if (currentExternalProgram != undefined) {
	currentAudioState = currentExternalProgram.takeAudio;
	currentFullscreenState = currentExternalProgram.fullscreen;
	currentControlLevel = currentExternalProgram.controlLevel;
    }
    
    if (progObject != undefined) {
	newAudioState = progObject.takeAudio;
	newFullscreenState = progObject.fullscreen;
	newControlLevel = progObject.controlLevel;
    }
    
    // Handle audio player class
    if (currentAudioState && !newAudioState) {
	audio_player_class.restore();
    } else if(newAudioState && !currentAudioState) {
	audio_player_class.setOwner(this);
	audio_player_class.dispose();
    }
    
    if (currentFullscreenState && !newFullscreenState) {
	// Show ostbox again
	screen.visible=true;
	showMain();
    } else if(newFullscreenState && !currentFullscreenState) {
	// Hide ostbox
	hideMain();
	screen.visible=false;
    }
    
    debug("currentControlLevel is "+currentControlLevel+"\n");
    debug("newControlLevel is "+newControlLevel+"\n");
    if (currentControlLevel != newControlLevel) {
	setInputLevel(newControlLevel);
    }
}

